import click
import logging
from pathlib import Path

import snex

log_dfmt = "%Y-%m-%d %H:%M:%S"
log_fmt = "%(asctime)s - %(levelname)-8s - %(name)s.%(funcName)s: %(message)s"
logging.basicConfig(level=logging.DEBUG, format=log_fmt, datefmt=log_dfmt)

logger = logging.getLogger(__name__)


@click.command()
@click.argument("base_path", default=".", type=click.Path(exists=True))
@click.argument("out_path", type=click.Path(), required=False)
@click.option("--config_file", type=click.Path(exists=True, dir_okay=False))
@click.option("-e", "--echo", count=True)
def main(base_path, out_path, config_file, echo):
    base_path = Path(base_path)
    if out_path is not None:
        out_path = Path(out_path)

    snippets = snex.extract(base_path, out_path, config_file)

    if echo > 0:
        for s in snippets:
            print("\t".join([str(entry) for entry in s[:echo]]))


if __name__ == "__main__":
    main()



__version__ = "2021.8.21"

import logging
from pyhocon import ConfigFactory
import snex.core as core
import snex.util as util
from pathlib import Path

logger = logging.getLogger(__name__)


def extract(base_path=None, out_path=None, config_file=None):
    if base_path is None:
        base_path = Path()

    if config_file is None:
        config_file = base_path / "snex.conf"

    processed_snippets = []
    logger.info(f"using config file {str(config_file)}")
    conf_root = ConfigFactory.parse_file(config_file)
    no_snippets = True
    for conf_name, conf in core.get_configs(conf_root):
        logger.info(f"processing config {conf_name}")

        if out_path is None:
            out_path = base_path / Path(conf["output_path"])
        logger.info(f"working dir: {str(base_path)}")
        logger.info(f"output dir: {str(out_path)}")
        out_path.mkdir(exist_ok=True, parents=True)

        out_suffix = conf.get("output_ext", None)
        if out_suffix is not None:
            logger.warning("config option 'output_ext' is deprecated, use 'output_suffix' instead")
        else:
            out_suffix = conf["output_suffix"]

        out_prefix = conf.get("output_prefix", f"{conf_name}-")

        snippets = (
            snippet
            for f in util.list_paths(conf, base_path)
            for snippet in core.extract_from_path(f, conf)
        )
        for snippet in snippets:
            no_snippets = False
            dst = out_path / f"{out_prefix}{snippet.name}{out_suffix}"
            origin = str(Path(snippet.origin).relative_to(base_path))
            logger.info(f"{origin}:{snippet.name} -> {dst}")
            processed_snippets.append((dst, snippet.name, origin, snippet.line_number))
            res = core.render_snippet(conf["output_template"], {**conf, **snippet.params}, snippet.body)
            dst.write_text(res)
    if no_snippets:
        logger.info("no snippets found")
    return processed_snippets



from pyhocon import ConfigFactory, ConfigTree
from itertools import dropwhile
from pathlib import Path
import logging
import re
import requests as rq

from yaml import load, dump

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

logger = logging.getLogger(__name__)


VALID_NAME_RE = r"[^-0-9A-Za-z_.%~äßöüÄ§ÖÜ€áàâãéêíóôõúçÁÀÂÃÉÊÍÓÔÕÚÇ]+"

# https://github.com/django/django/blob/stable/1.3.x/django/core/validators.py#L45

url_regex = re.compile(
    r"^(?:http|ftp)s?://"  # http:// or https://
    r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # domain...
    r"localhost|"  # localhost...
    r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
    r"(?::\d+)?"  # optional port
    r"(?:/?|[/?]\S+)$",
    re.IGNORECASE,
)

value = "http://gwil.de"


def is_url(value):
    return bool(url_regex.search(str(value)))


def construct_params(snippet_param_match, path, lnum):
    data = re.split(r"\s+", snippet_param_match.strip(), maxsplit=1)
    params = {"path": str(path), "lnum": lnum}
    if data and data[0]:
        params["name"] = data[0]
    else:
        params["name"] = f"{str(path)}-{lnum}"

    extra = load("{ " + data[1] + " }", Loader=Loader) if len(data) > 1 else {}
    x = {**extra, **params}
    return x


def read_path(path):
    if is_url(path):
        res = rq.get(path)
        for idx, line in enumerate(res.text.splitlines()):
            yield (idx, line)
    else:
        with open(path, "r") as fd:
            for idx, line in enumerate(fd):
                yield (idx, line)


def list_paths(conf, base_path):
    if "path" in conf:
        if is_url(conf["path"]):
            yield conf["path"]
        else:
            p = Path(conf["path"])
            if not p.is_absolute:
                p = base_path / p
            if p.is_file:
                yield p
    else:
        for f in find_files(base_path / conf["root"], conf["glob"]):
            yield f


# :snippet def-sanitize-for-filename
def sanitize_for_file_name(v):
    return re.sub(VALID_NAME_RE, "-", v)


# :endsnippet


def sanitize_params(params, valid_keys):
    if not ("name" in params and params["name"]):
        raise KeyError("name key not set")
    params["name"] = sanitize_for_file_name(params["name"])
    return {k: v for k, v in params.items() if k in valid_keys}


def find_files(root, glob):
    logger.info(f"{root} -> {glob}")
    for p in Path(root).glob(glob):
        if p.is_file():
            yield p


def det_indent_lvl(lines):
    return min([len(x) - len(x.lstrip()) for x in lines if x])


def dropwhile_right(pred, seq):
    return reversed(list(dropwhile(pred, reversed(seq))))


def merge_with_default_conf(conf, default_conf=None, global_default=None):
    if global_default is None:
        global_default = {}
    if default_conf is None:
        default_conf = ConfigTree()

    tmp = ConfigTree.merge_configs(ConfigFactory.from_dict(global_default), default_conf, copy_trees=True)
    return ConfigTree.merge_configs(tmp, conf)


def is_empty(x):
    return not bool(x)



import logging
import pystache
import re
from itertools import dropwhile
import snex.util as util

logger = logging.getLogger(__name__)

DEFAULT = {
    # :snippet global-default-config lang: python
    "output_template": "```{{lang}}\n{{{snippet}}}\n```\n",
    "valid_param_keys": ["name", "lang", "lnum"],
    "output_path": "extracted",
    "line_prefix": "",
    "comment_prefix": "# ",
    "comment_suffix": "",
    "snippet_start": ":snippet",
    "snippet_end": ":endsnippet",
    "cloak_start": ":cloak",
    "cloak_end": ":endcloak",
    "output_suffix": ".md",
    # :endsnippet
}


class Snippet:
    @classmethod
    def from_raw_data(cls, params, data, origin=None, prefix=""):
        (head,) = data[:1]
        (tail,) = data[-1:]
        body = data[1:-1]

        idlvl = util.det_indent_lvl(body)

        body = (line[idlvl:] for line in body)
        body = dropwhile(util.is_empty, body)
        body = util.dropwhile_right(util.is_empty, list(body))
        body = (prefix + line for line in body)

        return cls(params=params, head=head.lstrip(), body=list(body), tail=tail.lstrip(), origin=origin)

    @property
    def name(self):
        return self.params["name"]

    @property
    def line_number(self):
        return self.params.get("lnum", -1)

    def __init__(self, params=None, head=None, body=None, tail=None, origin=None):
        self.params = params
        self.head = head
        self.body = body
        self.tail = tail
        self.origin = origin

    def __repr__(self):
        return f"Snippet({self.params!r}, {self.head!r}, {self.body!r}, {self.tail!r}, {self.origin!r})"


def render_snippet(template, params, body):
    return pystache.render(template, {**params, **{"snippet": "\n".join(body)}})


def get_configs(conf):
    configs = conf["config"]
    default = configs["default"] if "default" in configs else None
    for name in [n for n in configs if not n == "default"]:
        c = util.merge_with_default_conf(configs[name], default, global_default=DEFAULT)
        # prevent defining a global name for all snippets in the config
        c.pop("name", None)
        yield (name, c)


def extract_from_path(f, conf):
    comment_prefix = re.escape(conf["comment_prefix"])
    comment_suffix = re.escape(conf["comment_suffix"])

    line_prefix = conf["line_prefix"]

    cloak_start = re.escape(conf["cloak_start"])
    cloak_end = re.escape(conf["cloak_end"])
    cloak_start_re = f"^\\s*{comment_prefix}{cloak_start}{comment_suffix}$"
    cloak_end_re = f"^\\s*{comment_prefix}{cloak_end}{comment_suffix}$"

    snippet_start = re.escape(conf["snippet_start"])
    snippet_end = re.escape(conf["snippet_end"])
    snippet_start_re = f"^\\s*{comment_prefix}{snippet_start}(.*){comment_suffix}$"
    snippet_end_re = f"^\\s*{comment_prefix}{snippet_end}{comment_suffix}$"

    snippets = []

    in_snippet = False
    cloaked = False
    data = []
    params = {}

    for idx, line in util.read_path(f):
        lnum = idx + 1
        if re.search(cloak_end_re, line):
            cloaked = False
            continue
        if re.search(cloak_start_re, line):
            cloaked = True
        if cloaked:
            continue

        if match := re.search(snippet_start_re, line):
            try:
                params = util.construct_params(match.group(1), f, lnum)
                params = util.sanitize_params(params, conf["valid_param_keys"])
            except Exception as ex:
                logger.error(f"could not parse snippet params: {line} in file {f}:{lnum}")
                raise ex
            in_snippet = True
        if not in_snippet:
            continue

        data.append(line.rstrip("\n"))
        if re.search(snippet_end_re, line):
            in_snippet = False
            snippets.append(Snippet.from_raw_data(params, data, origin=f, prefix=line_prefix))
            data = []
            params = {}
    return snippets



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['snex', 'snex.cli']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'pyhocon>=0.3.57,<0.4.0',
 'pystache>=0.5.4,<0.6.0',
 'pyyaml>=5.4.1,<6.0.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['snex = snex.cli:main']}

setup_kwargs = {
    'name': 'snex',
    'version': '2021.8.21',
    'description': 'snex - snippet extractor',
    'long_description': '# snex - snippet extractor\n\n[(Accompanying blog post)](https://bargsten.org/wissen/snex/)\n\nExtract snippets for blog posts or examples.\n\nHow to use\n\n## Installation\n\n    pip install snex\n\n## Tag Snippets\n\nLet\'s assume that you have a project in `/path/to/your/project`. You navigate to the\nregion where you want to extract a snippet and tag it as follows (`# ` is regarded as\ncomment prefix):\n\n    # :snippet snippet-name-without-whitespace\n\n    def foobar():\n       doit()\n    foobar()\n\n    # :endsnippet\n\n- Empty lines after the start and before the end are removed.\n- _A snippet name is mandatory._\n- The snippet `name` is sanitized to prevent malicious code to overwrite arbitrary files\n  on your system.\n\n### Advanced snippet tagging\n\nYou can also overwrite the `lang` config to use a different language for this snippet.\n\n```\n# :snippet snippet-name-without-whitespace lang: scala\n```\n\nEverything after the snippet name is parsed as YAML dict:\n`{ $text_after_snippet_name }`, e.g. `lang: scala, other_param: "hello world"` is parsed\nas `{ lang: scala, other_param: "hello world" }` YAML.\n\nThis means that you can also customise your parameter substitutions with a config like:\n\n````\nconfig {\n  default {\n    "output_template": "```{{lang}} - {{other_param}}\\n{{{snippet}}}\\n```\\n",\n    "valid_param_keys": [ "lang", "name", "other_param" ]\n    ...\n  }\n}\n````\n\nThe output template is parsed as [mustache template](https://mustache.github.io/).\n\n## Setup\n\ncreate a snex.conf in the root directory of a project you want to create snippets from:\n\n    config {\n      default {\n        output_path: "snippets"\n        comment_prefix: "# "\n        comment_suffix: ""\n      }\n\n      src {\n        lang: "python"\n        root: "src"\n        glob: "**/*.py"\n      }\n      github {\n        comment_prefix: "# "\n        lang: "python"\n        path: "https://raw.githubusercontent.com/jwbargsten/snex/master/src/snex/core.py"\n      }\n    }\n\nThe config syntax is\n[HOCON](https://github.com/typesafehub/config/blob/master/HOCON.md), under the hood\n[pyhocon](https://github.com/chimpler/pyhocon).\n\nYou have 3 layers of settings in a section:\n\n1.  [the global default config](docs/snippets/src-global-default-config.md) in\n    `docs/snippets/global-default-config.md`\n2.  the config section `default` in your `snex.conf` file (which overwrites the global\n    default).\n3.  the specific config section in your `snex.conf` (the section name is prefix for\n    snippets `default` is reserved.). The configuration in a specific section overwrites\n    the default section which overwrites the global default config.\n\n## Run\n\nYou created a `/path/to/your/project/snex.conf` like described in the previous topic.\n\n### From the project directory\n\n    cd /path/to/your/project\n    snex\n\nThis will read `snex.conf` in the current directory and dump the snippets into the\nconfigured `output_path`.\n\n### From a different directory\n\n    snex /path/to/your/project\n\nThis will read `/path/to/your/project/snex.conf` and dump the snippets into the\nconfigured `output_path`.\n\n### From a different directory to a different snippet output directory\n\n    snex /path/to/your/project /path/custom/snippet/output/dir\n\nThis will read `/path/to/your/project/snex.conf` and dump the snippets into\n`/path/custom/snippet/output/dir`.\n\n**TAKE CARE**\n\nThis invocation will overwrite the output dir of all defined config sections. Which\nmeans that all snippets are dumped into the same directory.\n\n## Caveats (or features)\n\n- Snippets are overwritten without confirmation. This makes it easy to update\n  everything, but you have to take care that you will not overwrite stuff you want to\n  keep.\n',
    'author': 'Joachim Bargsten',
    'author_email': 'jw@bargsten.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jwbargsten/snex',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
